@@ -1,6 +1,6 @@
 import os, json
 from typing import List, Tuple, Type, Union, Literal, Dict, Optional, Callable
-from langchain_core.prompts import ChatPromptTemplate, HumanMessagePromptTemplate, PromptTemplate, SystemMessagePromptTemplate
+from ..prompts import ChatPromptTemplate, HumanMessagePromptTemplate, PromptTemplate, SystemMessagePromptTemplate
 from ..GenerativeModel.generativeModels import MASGenerativeModel
 from ..Agents.singular_agent import Agent
 from ..pydanticModels.AnswerModel import answermodel
@@ -124,9 +124,10 @@ class AgentManager:
         elif 'all' in data:
             return data['all']
 
-    def create_agent(self, agent_name: str, tools: List[object], agent_details: AgentDetails, 
-                 memory_order: int = 10, long_context: bool = True,long_context_order: int = 20, shared_memory_order: int = 10, 
-                 plan: bool = False,temperature=0.2,context_callable:Optional[Callable]=None, **kwargs):
+    def create_agent(self, agent_name: str, tools: List[object], agent_details: AgentDetails,
+                 memory_order: int = 10, long_context: bool = True,long_context_order: int = 20, shared_memory_order: int = 10,
+                 plan: bool = False,temperature=0.2,context_callable:Optional[Callable]=None,retain_messages_order: int = 10,
+                 max_tool_output_words: int = 3000, **kwargs):
         """Create and register a new agent in the AgentManager.
 
         Args:
@@ -139,6 +140,8 @@ class AgentManager:
             shared_memory_order (int, optional): Shared memory size for components. Defaults to 10.
             plan (bool, optional): Include planner if True. Defaults to False.
             context_callable (Optional[Callable]): Callable that uses user input to give more context to the llm during inference.
+            retain_messages_order (int, optional): Number of past interactions to keep in memory for an agent's internal state across multiple queries. Defaults to 10.
+            max_tool_output_words (int, optional): Maximum number of words from tool output to include in LLM prompts. Defaults to 3000.
             
             **kwargs: Additional keyword arguments.  Can include:
                 - `config_dict` (dict, optional): A dictionary specifying memory order overrides for individual LLMs.
@@ -182,10 +185,13 @@ class AgentManager:
         tool_mapping = {tool.name: tool for tool in tools}
         AnswerFormat = answermodel(tool_names=list(tool_mapping.keys()) + ['None'], tools=tools)
 
+        # Cache JSON schema once at agent creation and attach to the model class
+        AnswerFormat._cached_schema = AnswerFormat.model_json_schema()
+
         # Initialize LLM models
         model_config = self._load_model_config(agent_name)
-        llm_args = {"temperature": temperature, "memory_order": memory_order, 
-                    "extra_context": self.context, 
+        llm_args = {"temperature": temperature, "memory_order": memory_order,
+                    "extra_context": self.context,
                     "long_context": long_context,
                     "long_context_order":long_context_order,
                     "chat_log":self.chat_log,
@@ -196,33 +202,85 @@ class AgentManager:
         if kwargs.get('in_memory_store'):
             llm_args["memory_store"] = kwargs['in_memory_store']
             llm_args['k'] = kwargs.get('top_k')
-        
-        
-        def override_config(component, llm_args, memory_order, long_context_order,temperature, **kwargs):
-            temp_args=llm_args.copy()
+
+
+        def extract_model_params(component_config):
+            """
+            Extract ALL model-specific parameters from component config.
+
+            This function is intentionally simple and scalable:
+            - Extracts ALL parameters except core ones (model_name, category)
+            - No hardcoded parameter lists
+            - BaseGenerativeModel handles filtering and mapping automatically
+
+            Users should refer to wrapper docstrings for available parameters:
+            - ChatGoogleGenerativeAI: See docstring for Gemini parameters
+            - ChatOpenAI: See docstring for OpenAI parameters
+            """
+            params = {}
+
+            # Core parameters that should NOT be passed as kwargs
+            core_params = {'model_name', 'category'}
+
+            # Extract ALL other parameters from component config
+            for key, value in component_config.items():
+                if key not in core_params:
+                    params[key] = value
+
+            return params
+
+
+        def override_config(component, llm_args, memory_order, long_context_order, temperature, **kwargs):
+            temp_args = llm_args.copy()
+
+            # Extract parameters from model_config.json for this component
+            component_params = extract_model_params(model_config[component])
+            temp_args.update(component_params)
+
+            # Override with config_dict if provided (highest priority)
             if "config_dict" in kwargs:
                 config_dict = kwargs["config_dict"]
-                temp_args["temperature"] = config_dict.get(f"{component}_temperature",temperature)
+
+                # Core MASAI parameters (always handled)
+                temp_args["temperature"] = config_dict.get(f"{component}_temperature", temp_args.get("temperature", temperature))
                 temp_args["memory_order"] = config_dict.get(f"{component}_memory_order", memory_order)
                 temp_args["long_context_order"] = config_dict.get(f"{component}_long_context_order", long_context_order)
+                temp_args['streaming'] = config_dict.get(f"{component}_streaming", self.streaming)
+                temp_args['streaming_callback'] = config_dict.get(f"{component}_streaming_callback", self.streaming_callback)
+
+                # Extract ALL other parameters prefixed with component name
+                # This is scalable - no hardcoded parameter lists
+                # Users should refer to wrapper docstrings for available parameters
+                component_prefix = f"{component}_"
+                core_keys = {'temperature', 'memory_order', 'long_context_order', 'streaming', 'streaming_callback'}
+
+                for config_key, value in config_dict.items():
+                    if config_key.startswith(component_prefix):
+                        # Extract parameter name (remove component prefix)
+                        param_name = config_key[len(component_prefix):]
+
+                        # Skip core parameters (already handled above)
+                        if param_name not in core_keys:
+                            temp_args[param_name] = value
+
             return temp_args
         
-        llm_router_args = override_config("router", llm_args, memory_order, long_context_order,temperature)
+        llm_router_args = override_config("router", llm_args, memory_order, long_context_order,temperature, **kwargs)
         llm_router = MASGenerativeModel(model_config["router"]["model_name"], category=model_config["router"]["category"], prompt_template=chat_prompts[0], **llm_router_args)
 
-        llm_evaluator_args = override_config("evaluator", llm_args, memory_order, long_context_order,temperature)
+        llm_evaluator_args = override_config("evaluator", llm_args, memory_order, long_context_order,temperature, **kwargs)
         llm_evaluator = MASGenerativeModel(model_config["evaluator"]["model_name"], category=model_config["evaluator"]["category"], prompt_template=chat_prompts[1], **llm_evaluator_args)
 
-        llm_reflector_args = override_config("reflector", llm_args, memory_order, long_context_order,temperature)
+        llm_reflector_args = override_config("reflector", llm_args, memory_order, long_context_order,temperature, **kwargs)
         llm_reflector = MASGenerativeModel(model_config["reflector"]["model_name"], category=model_config["reflector"]["category"], prompt_template=chat_prompts[2], **llm_reflector_args)
 
         if plan:
-            llm_planner_args = override_config("planner", llm_args, memory_order, long_context_order, temperature)
+            llm_planner_args = override_config("planner", llm_args, memory_order, long_context_order, temperature, **kwargs)
             llm_planner = MASGenerativeModel(model_config["planner"]["model_name"], category=model_config["planner"]["category"], prompt_template=chat_prompts[3], **llm_planner_args)
         else:
             llm_planner = None
 
-        agent = Agent(agent_name, llm_router, llm_evaluator, llm_reflector, llm_planner, tool_mapping, AnswerFormat, self.logging, shared_memory_order=shared_memory_order)
+        agent = Agent(agent_name, llm_router, llm_evaluator, llm_reflector, llm_planner, tool_mapping, AnswerFormat, self.logging, shared_memory_order=shared_memory_order,retain_messages_order=retain_messages_order, max_tool_output_words=max_tool_output_words)
         self.agents[agent_name.lower()] = agent
         self.agent_prompts[agent_name.lower()] = system_prompt
     def _compile_agents(self,type='decentralized',agent_context:dict=None):
@@ -281,7 +339,54 @@ class AgentManager:
     def list_agents(self) -> List[str]:
         """List all registered agents."""
         return list(self.agents.keys())
-    
-    
 
+    def cleanup(self) -> None:
+        """
+        Cleanup all resources held by AgentManager.
 
+        This method is called when AgentManager is removed from cache (after 30 min TTL).
+        It ensures all agents, LLM instances, and their memory are properly freed.
+        """
+        try:
+            # Clear all agents and their resources
+            for agent_name, agent in self.agents.items():
+                # Clear agent's retained state
+                if hasattr(agent, 'retained_state'):
+                    agent.retained_state = None
+
+                # Clear LLM chat histories
+                if hasattr(agent, 'llm_router') and agent.llm_router:
+                    if hasattr(agent.llm_router, 'chat_history'):
+                        agent.llm_router.chat_history.clear()
+
+                if hasattr(agent, 'llm_evaluator') and agent.llm_evaluator:
+                    if hasattr(agent.llm_evaluator, 'chat_history'):
+                        agent.llm_evaluator.chat_history.clear()
+
+                if hasattr(agent, 'llm_reflector') and agent.llm_reflector:
+                    if hasattr(agent.llm_reflector, 'chat_history'):
+                        agent.llm_reflector.chat_history.clear()
+
+                if hasattr(agent, 'llm_planner') and agent.llm_planner:
+                    if hasattr(agent.llm_planner, 'chat_history'):
+                        agent.llm_planner.chat_history.clear()
+
+                # Clear tool mapping
+                if hasattr(agent, 'tool_mapping'):
+                    agent.tool_mapping.clear()
+
+            # Clear agent registry
+            self.agents.clear()
+            self.agent_prompts.clear()
+
+            # Clear context
+            if self.context:
+                self.context.clear()
+
+        except Exception as e:
+            # Don't fail cleanup, just log if possible
+            print(f"Error during AgentManager cleanup: {e}")
+
+    def __del__(self):
+        """Destructor to ensure cleanup when AgentManager is garbage collected."""
+        self.cleanup()
