@@ -8,7 +8,9 @@ from dotenv import load_dotenv
 from ..GenerativeModel.baseGenerativeModel.basegenerativeModel import BaseGenerativeModel
 from ..Tools.logging_setup.logger import setup_logger
 from ..Tools.PARSERs.json_parser import parse_tool_input, parse_task_string
-from langgraph.graph import StateGraph # Only StateGraph needed here usually
+from ..langgraph.graph import StateGraph
+from ..langgraph.graph.state import CompiledStateGraph
+from ..Tools.utilities.streaming_events import emit_tool_call, emit_tool_output
 import inspect
 
 load_dotenv()
@@ -29,23 +31,36 @@ class State(TypedDict):
     passed_from: Optional[str] # Mark as Optional
     reflection_counter: int # Defaults set in Agent's initial_state
     tool_loop_counter: int # Defaults set in Agent's initial_state
+    tool_decided_by: Optional[str] # Track which node decided to use the current tool
+    current_question: str # Track the current question being processed
 
 class BaseAgent:
     _logger = None
 
-    def __init__(self, agent_name: str, logging: bool = True, shared_memory_order: int = 5, retain_messages_order: int = 20):
-        """Base class for agents with common functionality."""
+    def __init__(self, agent_name: str, logging: bool = True, shared_memory_order: int = 5, retain_messages_order: int = 30, max_tool_output_words: int = 3000):
+        """Base class for agents with common functionality.
+
+        Args:
+            agent_name: Name of the agent
+            logging: Enable logging
+            shared_memory_order: Number of messages to keep in shared memory
+            retain_messages_order: Number of messages to retain across executions
+            max_tool_output_words: Maximum number of words to include from tool output in LLM prompts (default: 3000)
+        """
         self.agent_name = agent_name.lower()
         self.logging = logging
         self.shared_memory_order = shared_memory_order
         self.retain_messages_order = retain_messages_order or 10 # Keep default if 20 not desired
+        self.max_tool_output_words = max_tool_output_words  # Configurable tool output limit
         self.app = None  # StateGraph workflow, to be set by subclasses
         self.graph = None
         # These will be set properly by the inheriting Agent class
         self.tool_mapping: Dict[str, Any] = {}
         self.pydanticmodel: Optional[Type[BaseModel]] = None
         self.agent_context: Optional[Dict[str, Any]] = None
-        # self.node = 'evaluator' # This seems less useful than state['current_node']
+
+        # NEW: Retained state for continuity across workflow executions
+        self.retained_state: Optional[Dict[str, Any]] = None
 
         if self.logging:
             # Use a shared logger instance across all agents if desired
@@ -58,6 +73,41 @@ class BaseAgent:
 
     # --- Synchronous Helper Methods ---
 
+    def _truncate_tool_output(self, tool_output: str, max_words: Optional[int] = None) -> str:
+        """
+        Truncate tool output to a maximum number of words.
+
+        Args:
+            tool_output: The tool output string to truncate
+            max_words: Maximum number of words (uses self.max_tool_output_words if None)
+
+        Returns:
+            Truncated tool output with truncation notice if truncated
+        """
+        if not tool_output or tool_output == 'N/A':
+            return tool_output
+
+        max_words = max_words or self.max_tool_output_words
+
+        # Convert to string if not already
+        tool_output_str = str(tool_output)
+
+        # Split into words
+        words = tool_output_str.split()
+
+        # Check if truncation is needed
+        if len(words) <= max_words:
+            return tool_output_str
+
+        # Truncate and add notice
+        truncated = ' '.join(words[:max_words])
+        truncation_notice = f"\n\n[... OUTPUT TRUNCATED: Showing first {max_words} words out of {len(words)} total words. Full output stored in state ...]"
+
+        if self.logger:
+            self.logger.info(f"Tool output truncated from {len(words)} words to {max_words} words for LLM prompt")
+
+        return truncated + truncation_notice
+
     async def gettoolinput(self, tool_input: Union[Dict, str], tool_name: str) -> Union[Dict, str]:
         """Parse and sanitize tool input based on the tool's schema."""
         # Ensure tool_mapping and tool exist before accessing args_schema
@@ -115,11 +165,19 @@ class BaseAgent:
         tool_input_raw = parsed_response.get('tool_input') # Assumes LLM returns 'tool_input'
 
         # Update previous/current node first
-        current_state['previous_node'] = current_state.get('current_node', None) # Record previous before overwrite
-        current_state['current_node'] = node
+        current_state['previous_node'] = node # The node we're entering
+        current_state['current_node'] = None  # Reset current
 
-        if tool_name and tool_name not in ["None", None]:
+        # Filter out Pydantic model name (happens with function_calling method)
+        # When using method="function_calling", OpenAI may return the model class name as tool
+        if tool_name and tool_name.upper() in ["ANSWERFORMAT", "ANSWER_FORMAT"]:
+            if self.logger: self.logger.debug(f"Filtered out Pydantic model name '{tool_name}' from tool field")
+            tool_name = None
+
+        if tool_name and tool_name not in ["None", None,'','null','none']:
             current_state["current_tool"] = tool_name
+            # Track which node decided to use this tool (for component context)
+            current_state["tool_decided_by"] = node
             # Parse tool input safely
             parsed_tool_input = await self.gettoolinput(tool_input_raw, tool_name)
 
@@ -144,6 +202,7 @@ class BaseAgent:
              # Clear tool info if no tool selected by LLM
              current_state["current_tool"] = None
              current_state["tool_input"] = None
+             current_state["tool_decided_by"] = None  # Clear decision maker
              current_state['tool_loop_counter'] = 0 # Reset counter if no tool
 
 
@@ -212,7 +271,7 @@ class BaseAgent:
         """Initiate the agent workflow asynchronously (to be overridden by subclasses)."""
         raise NotImplementedError("Subclasses must implement async initiate_agent")
 
-    def agentworkflow(self) -> StateGraph:
+    def agentworkflow(self) -> CompiledStateGraph:
         """Compile and return the agent's workflow graph (to be overridden by subclasses)."""
         raise NotImplementedError("Subclasses must implement agentworkflow")
 
@@ -228,11 +287,13 @@ class BaseAgent:
         Execute a tool based on the current state asynchronously,
         differentiating between async and sync based on the underlying tool func
         and trusting ainvoke if func inspection fails but ainvoke exists.
+
+        Emits streaming events for tool calls and outputs if streaming callback is set.
         """
+        # Import streaming utilities
         tool_name = state.get("current_tool")
         tool_input = state.get("tool_input")
 
-        state['previous_node'] = state.get('current_node')
         state['current_node'] = 'execute_tool'
 
         if not tool_name or tool_name == "None":
@@ -241,6 +302,10 @@ class BaseAgent:
             state['tool_output'] = "No tool to execute."
             if "messages" not in state: state["messages"] = []
             state["messages"].append({"role": "tool", "name": "system", "content": "No tool to execute."})
+
+            # Manual state update for early return
+            state['previous_node'] = 'execute_tool'
+            state['current_node'] = None
             return state
 
 
@@ -249,8 +314,13 @@ class BaseAgent:
             if self.logger: self.logger.error(f"Tool '{tool_name}' not found in tool_mapping.")
             state['tool_output'] = f"Error: Tool '{tool_name}' not found."
             state['satisfied'] = False
+            state['current_tool'] = None  # Clear invalid tool to prevent infinite loop
             if "messages" not in state: state["messages"] = []
             state["messages"].append({"role": "tool", "name": tool_name, "content": state['tool_output']})
+
+            # Manual state update for early return
+            state['previous_node'] = 'execute_tool'
+            state['current_node'] = None
             return state
 
 
@@ -258,6 +328,9 @@ class BaseAgent:
         if self.logger:
             self.logger.warning(f"Attempting to execute tool: {tool_name} with input: {tool_input}")
 
+        # Emit tool call event
+        await emit_tool_call(tool_name, tool_input, node=state.get('current_node'))
+
         result = None
         try:
             # Prepare input data (same logic as before)
@@ -338,35 +411,121 @@ class BaseAgent:
             result = f"Validation Error for tool '{tool_name}': {str(e)}. Input: {tool_input}. Ensure format matches requirements."
             if self.logger: self.logger.warning(f"Validation error executing tool '{tool_name}': {e}. Input: {tool_input}")
             state['satisfied'] = False
+            state['tool_output'] = result
+            state['current_tool'] = None  # Clear invalid tool to prevent infinite loop
+            if "messages" not in state: state["messages"] = []
+            state["messages"].append({"role": "tool", "name": tool_name, "content": state['tool_output']})
+
+            # Manual state update for early return
+            state['previous_node'] = 'execute_tool'
+            state['current_node'] = None
+            return state
         except Exception as e:
             # Catch the specific error if needed for debugging, otherwise general Exception
             result = f"Error executing tool '{tool_name}': {str(e)}. Input: {tool_input}"
             if self.logger:
                 self.logger.error(f"Unexpected error executing tool '{tool_name}': {e}", exc_info=True)
             state['satisfied'] = False
+            state['tool_output'] = result
+            state['current_tool'] = None  # Clear invalid tool to prevent infinite loop
+            if "messages" not in state: state["messages"] = []
+            state["messages"].append({"role": "tool", "name": tool_name, "content": state['tool_output']})
 
+            # Manual state update for early return
+            state['previous_node'] = 'execute_tool'
+            state['current_node'] = None
+            return state
 
-        # ... (no changes needed in state update/return logic) ...
-        if "messages" not in state: state["messages"] = []
-        if self.logger:
-            self.logger.warning("-------------------------------------Tool Output---------------------------------")
-            log_output = str(result)
-            # if len(log_output) > 500: log_output = log_output[:500] + "... (truncated)"
-            self.logger.warning(f"Tool: {tool_name}, Output: {log_output}")
-            self.logger.warning("-----------------------------------------------------------------------------------")
+        # Post-processing: Handle result and return_direct logic
+        # Wrap in try-except to catch any errors during result processing
+        try:
+            if "messages" not in state: state["messages"] = []
+            if self.logger:
+                self.logger.warning("-------------------------------------Tool Output---------------------------------")
+                log_output = str(result)
+                # if len(log_output) > 500: log_output = log_output[:500] + "... (truncated)"
+                self.logger.warning(f"Tool: {tool_name}, Output: {log_output}")
+                self.logger.warning("-----------------------------------------------------------------------------------")
 
-        state['tool_output'] = str(result)
-        state['passed_from'] = tool_name
-        state["messages"].append({"role": "tool", "name": tool_name, "content": state['tool_output']})
+            # Check for return_direct in three places with proper priority:
+            # Priority: Return Value > Argument > Decorator
+            # 1. Return value from tool execution (HIGHEST priority)
+            # 2. Dynamic return_direct from tool input argument (MEDIUM priority)
+            # 3. Tool decorator's return_direct attribute (LOWEST priority)
 
-        if hasattr(tool, 'return_direct') and tool.return_direct and state.get('satisfied', True):
-             if self.logger: self.logger.warning(f"Tool '{tool_name}' requested return_direct. Setting final answer.")
-             state.update({
-                 'current_tool': None, 'tool_input': None, 'answer': state['tool_output'],
-                 'reasoning': f"Result directly provided by tool '{tool_name}'.",
-                 'satisfied': True, 'delegate_to_agent': None,
-             })
-        elif 'satisfied' not in state:
-             state['satisfied'] = False
+            tool_decorator_return_direct = hasattr(tool, 'return_direct') and tool.return_direct
+            input_return_direct = None  # None means not provided
+            result_return_direct = None  # None means not provided
+            actual_result = result
 
+            # Check if tool_input contains return_direct argument
+            if isinstance(tool_input, dict) and 'return_direct' in tool_input:
+                input_return_direct = tool_input.get('return_direct')
+                if self.logger:
+                    self.logger.info(f"Tool '{tool_name}' has return_direct={input_return_direct} from tool_input argument")
+
+            # Check if result is a dict with return_direct key (tool decided internally during execution)
+            if isinstance(result, dict) and 'return_direct' in result:
+                result_return_direct = result.get('return_direct')
+                # Extract actual result (remove return_direct from output)
+                actual_result = {k: v for k, v in result.items() if k != 'return_direct'}
+                if self.logger:
+                    self.logger.info(f"Tool '{tool_name}' returned return_direct={result_return_direct} in result (runtime decision)")
+
+            # Implement proper priority: Return Value > Argument > Decorator
+            should_return_direct = False
+            source = None
+
+            if result_return_direct is not None:
+                # HIGHEST PRIORITY: Return value from function execution
+                should_return_direct = bool(result_return_direct)
+                source = "return value (runtime decision)"
+            elif input_return_direct is not None:
+                # MEDIUM PRIORITY: Argument passed to function
+                should_return_direct = bool(input_return_direct)
+                source = "input argument"
+            else:
+                # LOWEST PRIORITY: Decorator
+                should_return_direct = tool_decorator_return_direct
+                source = "decorator"
+
+            # Store the actual result (without return_direct metadata)
+            state['tool_output'] = str(actual_result)
+            state['passed_from'] = tool_name
+            state["messages"].append({"role": tool_name, "content": state['tool_output']})
+
+            # Emit tool output event
+            await emit_tool_output(tool_name, actual_result, node=state.get('current_node'))
+
+            if should_return_direct:
+                 if self.logger:
+                     self.logger.warning(f"Tool '{tool_name}' requested return_direct ({source}). Setting final answer.")
+                 state.update({
+                     'current_tool': None, 'tool_input': None, 'answer': state['tool_output'],
+                     'reasoning': f"Result directly provided by tool '{tool_name}'.",
+                     'satisfied': True, 'delegate_to_agent': None,
+                 })
+            elif 'satisfied' not in state:
+                 state['satisfied'] = False
+
+        except Exception as e:
+            # Catch any errors during result processing (e.g., str() conversion, dict operations, message append)
+            error_msg = f"Error processing tool result for '{tool_name}': {str(e)}. Raw result type: {type(result).__name__}"
+            if self.logger:
+                self.logger.error(f"Error during tool result processing for '{tool_name}': {e}", exc_info=True)
+
+            state['satisfied'] = False
+            state['tool_output'] = error_msg
+            state['current_tool'] = None
+            if "messages" not in state: state["messages"] = []
+            # Use try-except for message append in case state["messages"] is corrupted
+            try:
+                state["messages"].append({"role": "tool", "name": tool_name, "content": error_msg})
+            except Exception:
+                # If even this fails, create new messages list
+                state["messages"] = [{"role": "tool", "name": tool_name, "content": error_msg}]
+
+        # Manual state update to be consistent with other nodes (don't use _update_state as it expects LLM response)
+        state['previous_node'] = 'execute_tool'
+        state['current_node'] = None
         return state
\ No newline at end of file
