@@ -1,7 +1,7 @@
 import asyncio # Ensure asyncio is imported
-from langgraph.graph import END, StateGraph, START
+from ..langgraph.graph import END, StateGraph, START
 from typing import List, Dict, Any, Literal, TypedDict, Tuple, Union, Type, Optional, Generator, AsyncGenerator, Callable # Ensure all types needed are imported
-from langgraph.graph.state import CompiledStateGraph
+from ..langgraph.graph.state import CompiledStateGraph
 # Import the modified BaseAgent and other dependencies
 from .base_agent import BaseAgent, State # Assuming BaseAgent is in base_agent.py
 from pydantic import BaseModel, Field, ValidationError
@@ -17,12 +17,27 @@ class Agent(BaseAgent): # Inherit from the modified BaseAgent
                  tool_mapping: Optional[Dict[str, Any]] = None, # Use Any for Langchain tools, or specific Tool type
                  AnswerFormat: Optional[Type[BaseModel]] = None,
                  logging: bool = True, agent_context: Optional[Dict[str, Any]] = None, shared_memory_order: int = 5,
-                 retain_messages_order: int = 30, **kwargs):
+                 retain_messages_order: int = 30, max_tool_output_words: int = 3000, **kwargs):
 
         """Initialize an async agent with router-evaluator-reflector architecture and optional planner.
         Inherits async execute_tool from BaseAgent.
+
+        Args:
+            agent_name: Name of the agent
+            llm_router: LLM for router node
+            llm_evaluator: LLM for evaluator node
+            llm_reflector: LLM for reflector node
+            llm_planner: Optional LLM for planner node
+            tool_mapping: Dictionary of available tools
+            AnswerFormat: Pydantic model for structured output (with _cached_schema attribute)
+            logging: Enable logging
+            agent_context: Context dictionary for the agent
+            shared_memory_order: Number of messages to keep in shared memory
+            retain_messages_order: Number of messages to retain across executions
+            max_tool_output_words: Maximum number of words from tool output to include in LLM prompts (default: 3000)
+            **kwargs: Additional keyword arguments (e.g., max_tool_loop)
         """
-        super().__init__(agent_name, logging, shared_memory_order, retain_messages_order)
+        super().__init__(agent_name, logging, shared_memory_order, retain_messages_order, max_tool_output_words)
         self.llm_router = llm_router
         self.llm_evaluator = llm_evaluator
         self.llm_reflector = llm_reflector
@@ -62,7 +77,7 @@ class Agent(BaseAgent): # Inherit from the modified BaseAgent
         # Prepare arguments for the threaded call
         llm_call_args = {
             "prompt": prompt,
-            "output_structure": self.pydanticmodel,
+            "output_structure": self.pydanticmodel,  # Contains _cached_schema attribute
             "agent_context": self.agent_context,
             "agent_name": self.agent_name,
             "component_context": component_context or [],
@@ -77,6 +92,11 @@ class Agent(BaseAgent): # Inherit from the modified BaseAgent
             else:
                 parsed_response = await llm.generate_response_mas(**llm_call_args)
 
+            # Check if response is a string (error) instead of dict
+            if isinstance(parsed_response, str):
+                # LLM returned an error string, convert to proper error response
+                raise Exception(parsed_response)
+
             if self.logger:
                 log_answer = parsed_response.get('answer', 'N/A')
                 self.logger.info(f"Node {node_name} LLM response answer (truncated): {str(log_answer)[:config.truncated_response_length]}...")
@@ -116,7 +136,12 @@ class Agent(BaseAgent): # Inherit from the modified BaseAgent
     # --- Synchronous Condition/Helper Methods (Keep from original Agent) ---
 
     async def checkroutingcondition(self, state: State) -> Literal["continue", "end", "reflection"]:
-        """Determine the next step in the workflow (synchronous). Overrides BaseAgent.check_condition."""
+        """
+        Determine the next step in the workflow (synchronous). Overrides BaseAgent.check_condition.
+
+        NOTE: This is called AFTER _update_state, so state is already updated.
+        We save the state here for continuity across workflow executions.
+        """
         if self.logger:
             self.logger.info('----------------------------Deciding Node--------------------------------')
             self.logger.info(f"Checking routing: satisfied={state.get('satisfied')}, current_tool='{state.get('current_tool')}', reflection_counter={state.get('reflection_counter')}, tool_loop_counter={state.get('tool_loop_counter')}")
@@ -126,6 +151,10 @@ class Agent(BaseAgent): # Inherit from the modified BaseAgent
         reflection_counter = state.get('reflection_counter', 0)
         MAX_REFLECTIONS = config.MAX_REFLECTION_COUNT # Define max reflections
 
+        # NEW: Save state snapshot for continuity (before routing decision)
+        # This captures the current conversation state including messages, answer, etc.
+        self._save_state_snapshot(state)
+
         # End condition: Satisfied and no tool/delegation needed
         # Check for tool.return_direct case handled in execute_tool setting satisfied=True & tool=None
         if satisfied and current_tool in [None, "None", ""]:
@@ -149,6 +178,45 @@ class Agent(BaseAgent): # Inherit from the modified BaseAgent
         # This might need adjustment in the Reflector node logic.
         return "end"
 
+    def _save_state_snapshot(self, state: State) -> None:
+        """
+        Save a snapshot of the current state for continuity across workflow executions.
+
+        This is called in checkroutingcondition (after _update_state) to capture:
+        - Recent conversation messages
+        - Last answer
+        - Reasoning
+        - Any other relevant context
+        """
+        try:
+            # Get answer and ensure it's a string (not None)
+            last_answer = state.get("answer") or ""
+            last_reasoning = state.get("reasoning") or ""
+
+            # Create a lightweight snapshot (don't store everything)
+            self.retained_state = {
+                # Keep recent messages for context (limited by retain_messages_order)
+                "messages": state.get("messages", [])[-self.retain_messages_order:],
+                "last_answer": last_answer,
+                "last_reasoning": last_reasoning,
+                "satisfied": state.get("satisfied", False),
+                "tool_output": state.get("tool_output", None),
+                "current_tool": state.get("current_tool", None),
+                "tool_input": state.get("tool_input", None),
+                "tool_decided_by": state.get("tool_decided_by", None),
+                "previous_node": state.get("previous_node",None),
+                # Don't retain counters (fresh start for each query)
+            }
+
+            if self.logger:
+                self.logger.debug(f"State snapshot saved: {len(self.retained_state.get('messages', []))} messages, "
+                                f"last_answer length: {len(last_answer)}")
+        except Exception as e:
+            if self.logger:
+                self.logger.error(f"Error saving state snapshot: {e}", exc_info=True)
+            # Don't fail the workflow if snapshot fails
+            self.retained_state = None
+
 
     async def _tool_loop_warning_prompt(self, state: State) -> str:
         """Generate warning if stuck in a tool loop (synchronous)."""
@@ -159,9 +227,17 @@ class Agent(BaseAgent): # Inherit from the modified BaseAgent
 
     async def _format_node_prompt(self, state: State, node: str) -> str:
         """Format the prompt for a specific node (synchronous)."""
-        messages = state.get('messages', [])
-        tool_output = state.get('tool_output', 'N/A') # Default if no tool output yet
-        original_question = messages[0]['content'] if messages else "No original question found."
+        # Get tool output from state (full version stored in state)
+        tool_output_raw = state.get('tool_output', 'N/A')
+
+        # Truncate tool output for LLM prompt to prevent excessive token usage
+        # Full output remains in state for reference
+        tool_output = self._truncate_tool_output(tool_output_raw)
+
+        # Use current_question from state instead of messages[0]['content']
+        original_question = state.get('current_question', 'No original question found.')
+        tool_input = state.get('tool_input', 'N/A')
+
         # Safely access plan, format if exists
         plan_dict = state.get('plan')
         plan_str = ""
@@ -178,7 +254,8 @@ class Agent(BaseAgent): # Inherit from the modified BaseAgent
                     warning=warning,
                     original_question=original_question,
                     tool_output=tool_output,
-                    plan_str=plan_str
+                    plan_str=plan_str,
+                    tool_input=tool_input
                 )
 
         elif node == 'reflector':
@@ -187,7 +264,8 @@ class Agent(BaseAgent): # Inherit from the modified BaseAgent
                     original_question=original_question,
                     tool_output=tool_output,
                     plan_str=plan_str,
-                    reflection_count_display=reflection_count_display
+                    reflection_count_display=reflection_count_display,
+                    tool_input=tool_input
                 )
 
         elif node == 'planner':
@@ -201,7 +279,8 @@ class Agent(BaseAgent): # Inherit from the modified BaseAgent
              return ROUTER_NODE_PROMPT.format(
                     warning=warning,
                     original_question=original_question,
-                    plan_str=plan_str
+                    plan_str=plan_str,
+                    tool_output=tool_output,
                 )
 
 
@@ -213,23 +292,40 @@ class Agent(BaseAgent): # Inherit from the modified BaseAgent
 
     async def router(self, state: State) -> State:
         """Route the query to a tool or delegate asynchronously."""
+        state['current_node'] = 'router'
         if self.logger: self.logger.info("--- Entering Router Node (Async) ---")
         prev_node = state.get('previous_node')
-        # Simple context fetching (consider refining based on actual needs)
+
+        # Build component context with priority order
         component_context = []
+
+        # Priority 1: If coming directly from evaluator or reflector
         if prev_node == 'evaluator' and self.llm_evaluator:
              component_context = self.llm_evaluator.chat_history[-self.shared_memory_order:]
         elif prev_node == 'reflector' and self.llm_reflector:
              component_context = self.llm_reflector.chat_history[-self.shared_memory_order:]
+        elif prev_node == "execute_tool":
+            tool_decided_by = state.get('tool_decided_by')
+            if tool_decided_by == 'router' and self.llm_router:
+                pass
+            elif tool_decided_by == 'evaluator' and self.llm_evaluator:
+                component_context = self.llm_evaluator.chat_history[-self.shared_memory_order:]
+            elif tool_decided_by == 'reflector' and self.llm_reflector:
+                component_context = self.llm_reflector.chat_history[-self.shared_memory_order:]
         else:
-            component_context = state.get('messages', [])[-self.shared_memory_order:]
-        # Add other relevant contexts if needed
+            # Priority 2: Fallback to state messages
+            if prev_node != 'router': # Log only if not coming from router (common on first loop)
+                component_context = state.get('messages', [])[-self.shared_memory_order:]
+                # remove the last messae since it's same as query. We are providing query separately to prompt.
+                component_context = component_context[:-1]
+
 
         prompt = await self._format_node_prompt(state=state, node='router')
         return await self.node_handler(state, self.llm_router, prompt, component_context=component_context, node='router')
 
     async def evaluator(self, state: State) -> State:
         """Evaluate tool output asynchronously."""
+        state['current_node'] = 'evaluator'
         if self.logger: self.logger.info("--- Entering Evaluator Node (Async) ---")
         prev_node = state.get('previous_node')
         component_context = []
@@ -237,20 +333,25 @@ class Agent(BaseAgent): # Inherit from the modified BaseAgent
         # Or from the tool execution itself (passed via state['tool_output'])
         # Let's primarily use context from router/planner/reflector that *led* to the tool call
         if prev_node == 'execute_tool':
-             # Find node before execute_tool
-             messages = state.get('messages', [])
-             tool_node_index = -1
-             # Find the last tool message index
-             for i in range(len(messages) - 1, -1, -1):
-                 if messages[i].get("role") == "tool":
-                     tool_node_index = i
-                     break
-             if self.llm_router: component_context = self.llm_router.chat_history[-self.shared_memory_order:]
-             elif self.llm_planner: component_context = self.llm_planner.chat_history[-self.shared_memory_order:]
+             # Use context from the node that actually decided to use the tool
+             # Tool output is already included in the prompt via {tool_output}
+             tool_decided_by = state.get('tool_decided_by')
+             if tool_decided_by == 'router' and self.llm_router:
+                 component_context = self.llm_router.chat_history[-self.shared_memory_order:]
+             elif tool_decided_by == 'reflector' and self.llm_reflector:
+                 component_context = self.llm_reflector.chat_history[-self.shared_memory_order:]
+
+             # Fallback to router/planner if tool_decided_by is not set (backward compatibility)
+             elif not tool_decided_by:
+                 if self.llm_router: component_context = self.llm_router.chat_history[-self.shared_memory_order:]
+                 elif self.llm_planner: component_context = self.llm_planner.chat_history[-self.shared_memory_order:]
 
         elif prev_node == 'reflector' and self.llm_reflector: component_context = self.llm_reflector.chat_history[-self.shared_memory_order:]
         elif prev_node == 'router' and self.llm_router: component_context = self.llm_router.chat_history[-self.shared_memory_order:]
         elif prev_node == 'planner' and self.llm_planner: component_context = self.llm_planner.chat_history[-self.shared_memory_order:]
+        else:
+            if prev_node!= 'evaluator': # Log only if not coming from router (common on first loop)
+                component_context = state.get('messages', [])[-self.shared_memory_order:]
 
 
         prompt = await self._format_node_prompt(state=state, node='evaluator')
@@ -258,6 +359,7 @@ class Agent(BaseAgent): # Inherit from the modified BaseAgent
 
     async def reflection(self, state: State) -> State:
         """Reflect on progress and generate a final answer asynchronously."""
+        state['current_node'] = 'reflector'
         if self.logger: self.logger.info(f"--- Entering Reflection Node (Async) #{state.get('reflection_counter', 0) + 1} ---")
 
         # Increment reflection counter - handled by _update_state now? No, _update_state doesn't increment reflection counter. Do it here.
@@ -265,12 +367,21 @@ class Agent(BaseAgent): # Inherit from the modified BaseAgent
 
         prev_node = state.get('previous_node')
         component_context = []
-        # Gather context from relevant previous steps
+        # Gather context from relevant pr4evious steps
         if prev_node == 'router' and self.llm_router: component_context = self.llm_router.chat_history[-self.shared_memory_order:]
         elif prev_node == 'evaluator' and self.llm_evaluator: component_context = self.llm_evaluator.chat_history[-self.shared_memory_order:]
         elif prev_node == 'planner' and self.llm_planner: component_context = self.llm_planner.chat_history[-self.shared_memory_order:]
-        elif prev_node == 'execute_tool': # Reflecting after a tool failed evaluation
-            if self.llm_evaluator: component_context = self.llm_evaluator.chat_history[-self.shared_memory_order:]
+        elif prev_node == 'execute_tool':
+             # Use context from the node that actually decided to use the tool
+             # Tool output is already included in the prompt via {tool_output}
+             tool_decided_by = state.get('tool_decided_by')
+             if tool_decided_by == 'router' and self.llm_router:
+                 component_context = self.llm_router.chat_history[-self.shared_memory_order:]
+             elif tool_decided_by == 'evaluator' and self.llm_evaluator:
+                 component_context = self.llm_evaluator.chat_history[-self.shared_memory_order:]
+        else:
+            if prev_node!="reflector": # Log only if not coming from reflector (common on first loop)
+                component_context = state.get('messages', [])[-self.shared_memory_order:]
 
 
         prompt = await self._format_node_prompt(state=state, node='reflector')
@@ -281,6 +392,7 @@ class Agent(BaseAgent): # Inherit from the modified BaseAgent
 
     async def planner(self, state: State) -> State:
         """Plan tasks for complex queries asynchronously."""
+        state['current_node'] = 'planner'
         if self.logger: self.logger.info("--- Entering Planner Node (Async) ---")
         prev_node = state.get('previous_node')
         component_context = [] # Planner usually starts fresh or gets context from reflection if replanning
@@ -303,7 +415,7 @@ class Agent(BaseAgent): # Inherit from the modified BaseAgent
 
     # --- Workflow Compilation (Overrides BaseAgent placeholder) ---
 
-    def agentworkflow(self) -> StateGraph:
+    def agentworkflow(self) -> CompiledStateGraph:
         """Compile the agent's workflow graph with async nodes."""
         workflow = StateGraph(State) # type: ignore
 
@@ -336,8 +448,12 @@ class Agent(BaseAgent): # Inherit from the modified BaseAgent
 
         # --- Edges from intermediate nodes ---
 
-        # After executing a tool, always evaluate the result
-        workflow.add_edge("execute_tool", "evaluator")
+        # After executing a tool, decide next step (evaluator or end if return_direct)
+        workflow.add_conditional_edges("execute_tool", self.checkroutingcondition, {
+            "continue": "evaluator",  # Normal case: evaluate tool output
+            "reflection": "reflection",  # Tool execution suggests reflection needed
+            "end": END  # Tool has return_direct=True, go directly to end
+        })
 
         # After evaluating, decide next step based on satisfaction and tool needs
         workflow.add_conditional_edges("evaluator", self.checkroutingcondition, {
@@ -361,37 +477,63 @@ class Agent(BaseAgent): # Inherit from the modified BaseAgent
     async def initiate_agent(self, query: str, passed_from: Optional[str] = None, previous_node: Optional[str]= None) -> Dict:
         """
         Initiate the agent asynchronously and return the final result.
-        
+
         Args:
             query: The user query to process
             passed_from: Optional identifier of the component that passed the query
-            
+
         Returns:
             Dict: The final state containing the agent's response
         """
         if not self.app:
             raise ValueError("Agent workflow not compiled. Cannot initiate.")
         if self.logger: self.logger.debug(f"Agent: {self.agent_name}")
-        
+
         new_query = await self._sanitize_query(query)
-        
-        # Create initial state
-        initial_state = State(
-            messages=[{"role": "user", "content": new_query}],
-            current_tool="", tool_input=None, tool_output="", answer="",
-            satisfied=False, reasoning="", delegate_to_agent=None,
-            current_node='planner' if self.plan else 'router',
-            previous_node=previous_node, plan={}, passed_from=passed_from,
-            reflection_counter=0, tool_loop_counter=0
-        )
+
+        # NEW: Check if we have retained state from previous execution
+        if self.retained_state:
+            if self.logger:
+                self.logger.info(f"Restoring state from previous execution: "
+                               f"{len(self.retained_state.get('messages', []))} messages in history")
+
+            # Start with retained messages and add new query
+            initial_messages = self.retained_state.get("messages", []).copy()
+            initial_messages.append({"role": "user", "content": new_query})
+
+            # Create initial state with restored context
+            initial_state = State(
+                messages=initial_messages,
+                current_tool=self.retained_state.get("current_tool", ""), tool_input=self.retained_state.get("tool_input", None), tool_output=self.retained_state.get("tool_output", ""), answer=self.retained_state.get("last_answer", ""),
+                satisfied=False, reasoning="", delegate_to_agent=None,
+                current_node='planner' if self.plan else 'router',
+                previous_node=self.retained_state.get("previous_node", None), plan={}, passed_from=passed_from,
+                reflection_counter=0, tool_loop_counter=0, tool_decided_by=None,
+                current_question=new_query  # Track current question
+            )
+        else:
+            if self.logger:
+                self.logger.info("No retained state found, starting fresh")
+
+            # Create initial state (fresh start)
+            initial_state = State(
+                messages=[{"role": "user", "content": new_query}],
+                current_tool="", tool_input=None, tool_output="", answer="",
+                satisfied=False, reasoning="", delegate_to_agent=None,
+                current_node='planner' if self.plan else 'router',
+                previous_node=previous_node, plan={}, passed_from=passed_from,
+                reflection_counter=0, tool_loop_counter=0, tool_decided_by=None,
+                current_question=new_query  # Track current question
+            )
+
         configuration = {"recursion_limit": config.MAX_RECURSION_LIMIT}
-        
+
         try:
             # Execute fully and return the final state
             final_state = await self.app.ainvoke(initial_state, config=configuration)
             # Return the final state dictionary directly
             return dict(final_state)  # Ensure it's a plain dict
-        
+
         except Exception as e:
             if self.logger: self.logger.error(f"Error during agent invocation: {e}", exc_info=True)
             error_state = initial_state.copy()  # Start with initial state
@@ -408,34 +550,62 @@ class Agent(BaseAgent): # Inherit from the modified BaseAgent
     async def initiate_agent_astream(self, query: str, passed_from: Optional[str] = None, previous_node: Optional[str]= None) -> AsyncGenerator[Dict, None]:
         """
         Initiate the agent asynchronously with streaming updates.
-        
+
         Args:
             query: The user query to process
             passed_from: Optional identifier of the component that passed the query
-            
+
         Yields:
             Dict: State updates as the agent processes the query
         """
         if not self.app:
             raise ValueError("Agent workflow not compiled. Cannot initiate.")
         if self.logger: self.logger.debug(f"Agent: {self.agent_name}")
-        
+
         new_query = await self._sanitize_query(query)
-        
-        # Create initial state
-        initial_state = State(
-            messages=[{"role": "user", "content": new_query}],
-            current_tool="", tool_input=None, tool_output="", answer="",
-            satisfied=False, reasoning="", delegate_to_agent=None,
-            current_node='planner' if self.plan else 'router',previous_node=previous_node, plan={}, passed_from=passed_from,
-        reflection_counter=0, tool_loop_counter=0)
+
+        # NEW: Check if we have retained state from previous execution
+        if self.retained_state:
+            if self.logger:
+                self.logger.info(f"Restoring state from previous execution (streaming): "
+                               f"{len(self.retained_state.get('messages', []))} messages in history")
+
+            # Start with retained messages and add new query
+            initial_messages = self.retained_state.get("messages", []).copy()
+            initial_messages.append({"role": "user", "content": new_query})
+
+            # Create initial state with restored context
+            initial_state = State(
+                messages=initial_messages,
+                current_tool=self.retained_state.get("current_tool", ""), tool_input=self.retained_state.get("tool_input", None), tool_output=self.retained_state.get("tool_output", ""), answer=self.retained_state.get("last_answer", ""),
+                satisfied=False, reasoning="", delegate_to_agent=None,
+                current_node='planner' if self.plan else 'router',
+                previous_node=self.retained_state.get("previous_node", None), plan={}, passed_from=passed_from,
+                reflection_counter=0, tool_loop_counter=0, tool_decided_by=None,
+                current_question=new_query  # Track current question
+            )
+        else:
+            if self.logger:
+                self.logger.info("No retained state found, starting fresh (streaming)")
+
+            # Create initial state (fresh start)
+            initial_state = State(
+                messages=[{"role": "user", "content": new_query}],
+                current_tool="", tool_input=None, tool_output="", answer="",
+                satisfied=False, reasoning="", delegate_to_agent=None,
+                current_node='planner' if self.plan else 'router',
+                previous_node=previous_node, plan={}, passed_from=passed_from,
+                reflection_counter=0, tool_loop_counter=0, tool_decided_by=None,
+                current_question=new_query  # Track current question
+            )
+
         configuration = {"recursion_limit": config.MAX_RECURSION_LIMIT}
-    
+
         try:
             # Stream state updates
             async for state in self.app.astream(initial_state, config=configuration, stream_mode=[config.stream_mode]):
                 yield state
-        
+
         except Exception as e:
             if self.logger: self.logger.error(f"Error during streaming: {e}", exc_info=True)
             yield {
