@@ -1,7 +1,7 @@
 from pydantic import BaseModel
 from typing import Dict, List, Literal, Type, Tuple, Callable
 from typing import Optional
-from langchain.schema import Document
+from ..schema import Document
 from datetime import datetime
 from ..Memory.InMemoryStore import InMemoryDocStore
 from ..GenerativeModel.baseGenerativeModel.basegenerativeModel import BaseGenerativeModel
@@ -72,9 +72,9 @@ class GenerativeModel(BaseGenerativeModel):
 
 class MASGenerativeModel(BaseGenerativeModel):
     def __init__(
-        self, 
-        model_name: str, 
-        temperature: float, 
+        self,
+        model_name: str,
+        temperature: float,
         category: str,
         prompt_template = None,
         memory_order: int = 5,
@@ -103,6 +103,10 @@ class MASGenerativeModel(BaseGenerativeModel):
                 long_context_order (int, optional): The size or order of the long context when enabled. Controls how much historical data is considered. Defaults to 10.
                 chat_log (Optional[str], optional): File path to a chat log for loading past chat data as context. Defaults to None.
                 context_callable (Optional[Callable]): Callable that uses user input to give more context to the llm.
+                streaming (bool, optional): Enable or disable streaming of responses from
+                    the model. Defaults to False.
+                streaming_callback (Optional[Callable], optional): Async callback for streaming chunks. Defaults to None.
+                **kwargs: Additional keyword arguments to be passed to the model.
             Kwargs:
                 memory_store (InMemoryDocStore) : from masai.Memory.InMemoryStore import InMemoryDocStore and use it.
                 k (int, optional) : returns top k elements from memory store matching the query
@@ -113,7 +117,7 @@ class MASGenerativeModel(BaseGenerativeModel):
             category=category,
             memory_order=memory_order,
             prompt_template=prompt_template,
-            info=extra_context
+            info=extra_context if extra_context else {'USEFUL DATA': 'None'}
         )
 
         self.category = category
@@ -121,28 +125,31 @@ class MASGenerativeModel(BaseGenerativeModel):
         self.streaming = streaming
         self.streaming_callback = streaming_callback
         self.context_callable=context_callable
-        
-        
+
+
         self.long_context = long_context
         if self.long_context:
             self.llm_long_context = GenerativeModel(model_name=self.model_name,category=self.category,temperature=0.5,memory=False)
             self.context_summaries: List= []
         else:
             self.llm_long_context, self.context_summaries = None, None
-            
-            
+
+
         self.long_context_order = long_context_order
-        
+
         self.LTIMStore : InMemoryDocStore= kwargs.get('memory_store')
-            
+
         if self.LTIMStore and self.long_context:
             if not kwargs['memory_store']:
                 raise ValueError('InMemoryDocStore instance must be Provided.')
             else:
                 self.LTIMStore = kwargs['memory_store']
         else:
-            self.LTIMStore=None            
+            self.LTIMStore=None
         self.chat_log = chat_log
+
+        # Cache for performance optimization (only for info, not schema)
+        self._cached_formatted_info = None  # Will be set on first use
         
     async def _update_long_context(self, messages: List[Dict[str, str]]) -> List:
         """
@@ -184,6 +191,7 @@ class MASGenerativeModel(BaseGenerativeModel):
             
             self.logger.error(f"Error in long context summarization: {e}")
             raise e
+    
 
     async def _update_context_callable(self, query, role):
         if role.lower()=="user":
@@ -199,8 +207,8 @@ class MASGenerativeModel(BaseGenerativeModel):
 
 
     async def generate_response_mas(
-        self, 
-        prompt: str, 
+        self,
+        prompt: str,
         output_structure: Type[BaseModel],
         agent_context: Optional[dict] = None,
         agent_name: Optional[str] = "assistant",
@@ -209,10 +217,10 @@ class MASGenerativeModel(BaseGenerativeModel):
     ):
         """
         MAS-specific response generation with agent context support.
-        
+
         Args:
             prompt: The input prompt
-            output_structure: Pydantic model for structured output (AnswerFormat)
+            output_structure: Pydantic model for structured output (AnswerFormat with _cached_schema attribute)
             agent_context: Dictionary containing information about other agents
             agent_name: Name of the current agent
             component_context: Additional context messages from previous components
@@ -236,23 +244,36 @@ class MASGenerativeModel(BaseGenerativeModel):
         
         
 
-        # Prepare MAS-specific inputs
+        # # Prepare MAS-specific inputs
+        # start_formating_time = time.time()
+
+
+        # Cache formatted info if it hasn't changed
+        if self._cached_formatted_info is None or self._cached_formatted_info != str(list(self.info.items())):
+            self._cached_formatted_info = str(list(self.info.items())) if self.info else "None"
+
         mas_inputs = {
-            "useful_info": str([self.info.items() if self.info else None]),
+            "useful_info": self._cached_formatted_info,
             "current_time": datetime.now().strftime("%A, %B %d, %Y, %I:%M %p"),
             "question": prompt,
             "long_context": self.context_summaries if not in_memory_store_data else in_memory_store_data.extend(self.context_summaries),
             "history": self.chat_history,
-            "schema": output_structure.model_json_schema(),
+            "schema": getattr(output_structure, '_cached_schema', None) or output_structure.model_json_schema(),  # From output_structure._cached_schema
             "coworking_agents_info": agent_context if agent_context is not None else "NO AGENTS PRESENT"
         }
         
         try:
             # print("\n\n\n\n",self.prompt.format(**mas_inputs),"\n\n")
+            # print("END FORMAT TIME", time.time()-start_formating_time)
             # Use the prompt template with MAS-specific inputs
             if self.logger:
                 startapi=time.time()
-            response = (await self.model.with_structured_output(output_structure).ainvoke(
+                self.logger.debug(f"LLM Api request time {time.time()-startapi}")
+
+            # Use json_mode for OpenAI and Gemini for better reliability
+            structured_model = self._return_structured_model(prompt, output_structure)
+
+            response = (await structured_model.ainvoke(
                 self.prompt.format(**mas_inputs)
             )).model_dump()
             
@@ -260,13 +281,14 @@ class MASGenerativeModel(BaseGenerativeModel):
                 
                 
                 self.logger.debug(f"LLM Api response time {time.time()-startapi}")
-                # self.logger.info(response['answer'][:20])
+                self.logger.info(f"Returned Response : {response}")
             
-            # print(response)
-            # Update chat history with structured response
-            
-            if isinstance(response, dict) and 'answer' in response:
-                self.chat_history.append({'role': role, 'content': response['answer']})
+            # Update chat history with structured response and last question/prompt
+            self.chat_history.append({'role': role, 'content': prompt})
+            if isinstance(response, dict) and 'answer' in response and response['answer'] is not None:
+                self.chat_history.append({'role': agent_name, 'content': response['answer']})
+            elif isinstance(response, dict) and 'reasoning' in response and response['reasoning'] is not None:
+                self.chat_history.append({'role': agent_name, 'content': response['reasoning']})
             
             # print("returning response", time.time()-start)
             
@@ -289,11 +311,52 @@ class MASGenerativeModel(BaseGenerativeModel):
 
     
     async def _update_component_context(self, component_context, role, prompt):
+        # import time
+        # start_time = time.time()
+
+        # Extract tool output from current prompt once
+        tool_output_in_prompt = self._extract_tool_output_from_prompt(prompt)
+
+        if tool_output_in_prompt:
+            # Truncate overlapping tool outputs in component context messages
+            if component_context:
+                truncated_context = []
+                for message in component_context:
+                    if not isinstance(message, dict) or 'content' not in message:
+                        truncated_context.append(message)
+                        continue
+
+                    content = message.get('content', '')
+                    if content is None:
+                        content = ''
+
+                    truncated_message = message.copy()
+                    truncated_message['content'] = self._truncate_overlapping_tool_output(content, tool_output_in_prompt)
+                    truncated_context.append(truncated_message)
+                component_context = truncated_context
+
+            # Also truncate tool outputs in existing chat_history to avoid duplication
+            # This handles both tagged tool outputs AND untagged content that overlaps
+            # history_truncate_start = time.time()
+            self.chat_history = [
+                {'role': msg.get('role', ''), 'content': self._truncate_overlapping_tool_output(msg.get('content', '') or '', tool_output_in_prompt)}
+                if isinstance(msg, dict) else msg
+                for msg in self.chat_history
+            ]
+
+            # print(f"⏱️ CONTEXT FILTERING TIME: Total={time.time()-start_time:.4f}s | Component={len(component_context or [])} msgs | History={len(self.chat_history)} msgs | Time={time.time()-history_truncate_start:.4f}s")
+
         if component_context:
             self.chat_history.extend(component_context)
-            self.chat_history.append({'role': role, 'content': prompt})
-        else:
-            self.chat_history.append({'role': role, 'content': prompt})
+        # else:
+        #     self.chat_history.append({'role': role, 'content': prompt})
+
+
+
+
+
+
+
     async def _update_role(self, agent_name, kwargs):
         if 'passed_from' in kwargs:
             if kwargs['passed_from'] is not None:
@@ -352,9 +415,114 @@ class MASGenerativeModel(BaseGenerativeModel):
                 return None
         else:
             return None
+        
+    def _extract_tool_output_from_prompt(self, prompt: str) -> str:
+        """
+        Extract tool output content from prompt's <PREVIOUS TOOL OUTPUT START>...<END> tags.
+        Args:
+            prompt: The prompt that may contain tool output tags
+        Returns:
+            Extracted tool output content, or empty string if not found
+        """
+        import re
+        pattern = r'<PREVIOUS TOOL OUTPUT START>\n(.*?)\n<PREVIOUS TOOL OUTPUT END>'
+        match = re.search(pattern, prompt, re.DOTALL)
+        return match.group(1).strip() if match else ''
+
+    def _check_overlap(self, text1: str, text2: str, min_chunk_words: int = 20) -> bool:
+        """
+        Check if two texts have significant overlap by finding common word chunks.
+        Returns True if at least min_chunk_words consecutive words match.
+        """
+        if not text1 or not text2:
+            return False
+
+        words1 = text1.split()
+        words2 = text2.split()
+
+        if len(words1) < min_chunk_words or len(words2) < min_chunk_words:
+            # If either text is too short, check if shorter one is substring of longer
+            shorter = text1 if len(words1) < len(words2) else text2
+            longer = text2 if len(words1) < len(words2) else text1
+            return shorter in longer
+
+        # Check if any chunk of min_chunk_words from text1 appears in text2
+        for i in range(len(words1) - min_chunk_words + 1):
+            chunk = ' '.join(words1[i:i + min_chunk_words])
+            if chunk in text2:
+                return True
+
+        return False
+
+    def _truncate_overlapping_tool_output(self, content: str, tool_output_reference: str, max_words: int = 30) -> str:
+        """
+        Truncate tool output in content if it overlaps with reference.
+        Handles both tagged tool outputs and untagged content that contains tool output.
+
+        Args:
+            content: Message content that may contain tool output
+            tool_output_reference: The reference tool output from current prompt
+            max_words: Maximum number of words to keep in tool output
+        Returns:
+            Content with truncated tool output if overlap found, otherwise unchanged
+        """
+        if not content or not isinstance(content, str) or not tool_output_reference:
+            return content or ''
+
+        import re
+
+        # Clean reference (remove "..." for comparison)
+        reference_clean = tool_output_reference.rstrip('.')
+
+        # Pattern for tagged tool outputs
+        pattern = r'<PREVIOUS TOOL OUTPUT START>\n(.*?)\n<PREVIOUS TOOL OUTPUT END>'
+
+        def truncate_if_overlapping(match):
+            tool_output = match.group(1).strip()
+
+            # Check if already truncated
+            if tool_output.endswith('...'):
+                return match.group(0)
+
+            # Clean tool output
+            tool_output_clean = tool_output.rstrip('.')
+
+            # Check overlap in BOTH directions (handles truncated vs full)
+            has_overlap = (self._check_overlap(tool_output_clean, reference_clean) or
+                          self._check_overlap(reference_clean, tool_output_clean))
+
+            if has_overlap:
+                words = tool_output.split()
+                if len(words) <= max_words:
+                    return match.group(0)  # Already short enough
+
+                # Truncate
+                truncated = ' '.join(words[:max_words])
+                return f'<PREVIOUS TOOL OUTPUT START>\n{truncated}...\n<PREVIOUS TOOL OUTPUT END>'
+
+            return match.group(0)  # No overlap
+
+        # First, handle tagged tool outputs
+        modified_content = re.sub(pattern, truncate_if_overlapping, content, flags=re.DOTALL)
+
+        # Second, check if untagged content contains large chunks of tool output
+        # (e.g., LLM directly outputting tool results in its response)
+        if modified_content == content and '<PREVIOUS TOOL OUTPUT' not in content:  # No tagged outputs
+            # Check if content itself (without tags) overlaps with reference
+            content_clean = content.rstrip('.')
+            if self._check_overlap(content_clean, reference_clean, min_chunk_words=30):
+                # Large overlap found in untagged content - truncate the entire content
+                words = content.split()
+                if len(words) > max_words:
+                    truncated = ' '.join(words[:max_words])
+                    return f'{truncated}... [truncated - overlaps with tool output]'
+
+        return modified_content
+
+
     async def astream_response_mas(
-        self, 
-        prompt: str, 
+        self,
+        prompt: str,
         output_structure: Type[BaseModel],
         agent_context: Optional[dict] = None,
         agent_name: Optional[str] = "assistant",
@@ -363,10 +531,10 @@ class MASGenerativeModel(BaseGenerativeModel):
     ):
         """
         MAS-specific response generation with streaming support.
-        
+
         Args:
             prompt: The input prompt
-            output_structure: Pydantic model for structured output
+            output_structure: Pydantic model for structured output (AnswerFormat with _cached_schema attribute)
             agent_context: Dictionary containing information about other agents
             agent_name: Name of the current agent
             component_context: Additional context messages from previous components
@@ -380,24 +548,30 @@ class MASGenerativeModel(BaseGenerativeModel):
             await self._update_chat_history()
 
         role = await self._update_role(agent_name,kwargs)
-        await self._update_context_callable(query=prompt)
+        await self._update_context_callable(query=prompt, role=role)
         in_memory_store_data : Optional[List] = await self._handle_in_memory_store_search(k=kwargs.get('k'), prompt=prompt)
         await self._update_component_context(component_context=component_context, role=role, prompt=prompt)
 
         # Prepare MAS-specific inputs
+        # Extract cached schema from output_structure (attached at agent creation)
+
+        if self._cached_formatted_info is None or self._cached_formatted_info != str(list(self.info.items())):
+            self._cached_formatted_info = str(list(self.info.items())) if self.info else "None"
+
         mas_inputs = {
-            "useful_info": str([self.info.items() if self.info else 'None']),
+            "useful_info": self._cached_formatted_info,
             "current_time": datetime.now().strftime("%A, %B %d, %Y, %I:%M %p"),
             "question": prompt,
             "long_context": self.context_summaries if not in_memory_store_data else in_memory_store_data.extend(self.context_summaries),
             "history": self.chat_history,
-            "schema": output_structure.model_json_schema(),
+            "schema": getattr(output_structure, '_cached_schema', None) or output_structure.model_json_schema(),  # From output_structure._cached_schema
             "coworking_agents_info": agent_context if agent_context is not None else "No agents present"
         }
         
         try:
             # Create structured output model for streaming
-            structured_llm = self.model.with_structured_output(output_structure)
+            # Use json_mode for OpenAI and Gemini for better reliability
+            structured_llm= self._return_structured_model(prompt=prompt,output_structure=output_structure)
             
             # Final response to be returned
             final_response = None
@@ -420,12 +594,18 @@ class MASGenerativeModel(BaseGenerativeModel):
             # Convert final response to dict if it's a model
             if hasattr(final_response, 'model_dump'):
                 response_dict = final_response.model_dump()
+            elif hasattr(final_response, 'content'):
+                # AIMessage object - streaming failed to produce Pydantic model
+                raise ValueError(f"Streaming failed to produce structured output. Got AIMessage instead: {final_response.content[:200]}")
             else:
                 response_dict = final_response
                 
-            # Update chat history with the complete response
-            if isinstance(response_dict, dict) and 'answer' in response_dict:
-                self.chat_history.append({'role': role, 'content': response_dict['answer']})
+            # Update chat history with the complete response or last question/prompt
+            self.chat_history.append({'role': role, 'content': prompt})
+            if isinstance(response_dict, dict) and 'answer' in response_dict and response_dict['answer'] is not None:
+                self.chat_history.append({'role': agent_name, 'content': response_dict['answer']})
+            elif isinstance(response_dict, dict) and 'reasoning' in response_dict and response_dict['reasoning'] is not None:
+                self.chat_history.append({'role': agent_name, 'content': response_dict['reasoning']})
             
             return response_dict
             
